/*
 * Common display-related functions.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "display.h"



/* Return the line data position for the given pixel. */

int get_position(display_config_t *cfg, int x)
{
    int cell = x / cfg->cell_size, offset = x % cfg->cell_size;
    int pos = (cell / 2) * cfg->cell_size + offset;

    return cell % 2 ? pos + cfg->line_length / 2 : pos;
}

/* Provide a pattern to test the line data. */

void test_linedata(display_config_t *cfg)
{
    int x, y;
    uint8_t *linedata = cfg->framebuffer;

    for (y = 0; y < cfg->line_count; y++)
    {
        for (x = 0; x < cfg->line_length; x++)
        {
            /* Pixel: I0RRGGBB = Y0YYYYXX */

            linedata[get_position(cfg, x)] = (x % 2) ?
                    (((y / (cfg->line_count / 32)) & 0b1) << 7) |
                    (((y / (cfg->line_count / 16)) & 0b1111) << 2) |
                    ((x / (cfg->line_length / 4)) & 0b11) :
                0x00;
        }

        linedata += cfg->line_length;
    }
}

/* Copying from/to the display to/from a backing store. */

void copy_display(display_config_t *cfg, uint8_t *store, int width, int height,
                  int x, int y, int key, int to_display)
{
    int sx, sy, dx, dy;
    uint8_t *storeline = store,
            *displayline = cfg->framebuffer + y * cfg->line_length,
            pixel;

    for (sy = 0, dy = y; (sy < height) && (dy < cfg->line_count); sy++, dy++)
    {
        for (sx = 0, dx = x; (sx < width) && (dx < cfg->line_length); sx++, dx++)
        {
            if (to_display)
            {
                pixel = storeline[sx];
                if ((key < 0) || (pixel != key))
                    displayline[get_position(cfg, dx)] = pixel;
            }
            else
                storeline[sx] = displayline[get_position(cfg, dx)];
        }

        storeline += width;
        displayline += cfg->line_length;
    }
}
